﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Exchange.WebServices.Data;

namespace ConsoleApplication12
{
	class Program
	{
		static void Main(string[] args)
		{
			ExchangeService service = new ExchangeService();
			service.AutodiscoverUrl("your_email@address.com"); // TODO: Change this to the user's email address

			Folder commonViewsFolder = GetCommonViewsFolder(service);

			if (commonViewsFolder != null)
			{
				// TODO: Replace these with the actual calendar names loaded by the exchange connector.
				string[] calendarNames = new string[] { "Calendar", "Calendar2", "Calendar3", "Calendar4" };

				List<string> overlayCalendars;
				List<string> remainingCalendars;
				GetCalendarGroupInformation(service, commonViewsFolder, calendarNames, out overlayCalendars, out remainingCalendars);

				// TODO: Use the ordered collection of calendars here.
			}
		}

		private static void GetCalendarGroupInformation(ExchangeService service, Folder commonViewsFolder, string[] calendarNames, 
			out List<string> overlayCalendars, out List<string> remainingCalendars)
		{
			SortedList<byte[], string> overlayCalendarsSorted = new SortedList<byte[], string>(new CalendarSorter());
			SortedList<byte[], string> remainingCalendarsSorted = new SortedList<byte[], string>(new CalendarSorter());

			ItemView itemsView = new ItemView(20);
			itemsView.Traversal = ItemTraversal.Associated;

			while (true)
			{
				FindItemsResults<Item> items = commonViewsFolder.FindItems(itemsView);
				foreach (var item in items)
				{
					if (item.ItemClass != "IPM.Microsoft.WunderBar.Link")
						continue;

					if (calendarNames.Contains(item.Subject))
					{
						PropertySet set = new PropertySet(BasePropertySet.FirstClassProperties);
						set.Add(new ExtendedPropertyDefinition(0x684A, MapiPropertyType.Integer));
						set.Add(new ExtendedPropertyDefinition(0x684B, MapiPropertyType.Binary));
						item.Load(set);

						ExtendedProperty flagsProp = item.ExtendedProperties[0];
						ExtendedProperty orderProp = item.ExtendedProperties[1];

						int flags = (int)flagsProp.Value;

						if ((flags & 0x00100000) == 0)
							continue; // Not sure if this is correct.

						bool isInOverlay = (flags & 0x1000) != 0;
						byte[] relativeOrder = (byte[])orderProp.Value;

						if (isInOverlay)
						{
							if (overlayCalendarsSorted.Values.Contains(item.Subject) == false)
								overlayCalendarsSorted.Add(relativeOrder, item.Subject);
						}
						else
						{
							if (remainingCalendarsSorted.Values.Contains(item.Subject) == false)
								remainingCalendarsSorted.Add(relativeOrder, item.Subject);
						}
					}
				}

				if (items.NextPageOffset.HasValue == false)
				{
					overlayCalendars = new List<string>(overlayCalendarsSorted.Values);
					remainingCalendars = new List<string>(remainingCalendarsSorted.Values);
					return;
				}

				itemsView.Offset = items.NextPageOffset.Value;
			}
		}

		private static Folder GetCommonViewsFolder(ExchangeService service)
		{
			Folder rootFolder = Folder.Bind(service, WellKnownFolderName.Root);

			FolderView folderView = new FolderView(20);
			while (true)
			{
				FindFoldersResults results = rootFolder.FindFolders(folderView);

				foreach (var folder in results)
				{
					if (folder.DisplayName == "Common Views")
						return folder;
				}

				if (results.NextPageOffset.HasValue == false)
					return null;

				folderView.Offset = results.NextPageOffset.Value;
			}
		}

		private class CalendarSorter : Comparer<byte[]>
		{
			public override int Compare(byte[] x, byte[] y)
			{
				for (int index = 0; index < x.Length && index < y.Length; index++)
				{
					int result = x[index].CompareTo(y[index]);
					if (result != 0)
						return result;
				}

				return x.Length.CompareTo(y.Length);
			}
		}
	}
}
